package com.muzhi.tianhe.controller;

import com.alibaba.fastjson.JSONObject;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.aliwork.YDSearch;
import com.malk.server.common.McR;
import com.malk.service.aliwork.YDClient;
import com.malk.utils.UtilMap;
import com.muzhi.tianhe.entity.Tianhe;
import com.muzhi.tianhe.entity.vo.TianheDataVo;
import com.muzhi.tianhe.entity.vo.TianheQuery;
import com.muzhi.tianhe.exceptionhander.R;
import com.muzhi.tianhe.service.TbService;
import com.muzhi.tianhe.service.ThTbService;
import com.muzhi.tianhe.service.impl.TbApiService;
import com.muzhi.tianhe.util.UtilExcel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.http.HttpResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.Instant;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Author ZhangKan
 * @Data 15:03
 */

@Api(description = "TB接口")
@EnableScheduling
@CrossOrigin
@RestController
@RequestMapping("/tbservice")
public class TbController {

    @Autowired
    private TbService tbService;

    @Autowired
    private ThTbService thTbService;

    @Autowired
    private TbApiService tbApiService;

    //获取项目信息
    @Scheduled(cron = "0 0 0,12 * * ? ")
    @ApiOperation(value = "获取项目信息")
    @PostMapping("getXiangmu")
    public String getXiangmu(){
        tbService.asyncXm();
        return "success";
    }

    //获取项目状态
    @Scheduled(cron = "0 0 1 * * ? ")
    @ApiOperation(value = "获取项目状态")
    @PostMapping("syncState")
    public McR syncState(){
        thTbService.syncState();
        return McR.success();
    }

    // 同步某个项目的信息
    @GetMapping("syncStateByPid")
    public McR getCust(@RequestParam String projectId){
        thTbService.syncStateProject(projectId);
        return McR.success();
    }

    @GetMapping("syncProject")
    public String syncProject(String id){
        tbService.asyncXm(id);
        return "success";
    }

    //查询数据库中tianhe表的所有数据
    @ApiOperation(value = "查询数据库中tianhe表的所有数据")
    @GetMapping("getTianhe")
    public List<Tianhe> getTianhe(){
        List<Tianhe> tianheList = tbService.getTianhe();
        return tianheList;
    }

    //删除tianhe表中的所有数据
    @ApiOperation(value = "删除tianhe表中的所有数据")
    @PostMapping("removeTianhe")
    public R removeTianhe(){
        String result = tbService.removeTianhe();
        return R.ok().data("result", result);
    }

    //获取数据库中tianhe实体类表数据并放入vo对象进行前端接口调用展示数据
    @ApiOperation(value = "获取天合TB项目信息")
    @GetMapping("getTianheTB")
    public TianheDataVo getTianheTB(Integer page, Integer limit,
                                    TianheQuery tianheQuery){

//        @RequestBody是使用json传递数据，把json数据封装到对应的对象里面，注意：需要用Post提交方式，Get取不到值
//        (required = false)表示参数值可以为空
//        @ApiParam(name = "teacherQuery", value = "自定义讲师查询条件", required = true)
//        @RequestBody(required = false) EmployeeQuery employeeVoQuery

        TianheDataVo tianheTB = tbService.getTianheTB(page,limit,tianheQuery);
        return tianheTB;
    }

    //获取数据库中tianhe实体类表数据并导出
    @ApiOperation(value = "导出天合TB项目信息")
    @GetMapping("exportTianheTB")
    public void exportTianheTB(HttpServletResponse response,TianheQuery tianheQuery) throws IOException {
        TianheDataVo tianheTB = tbService.getTianheTB(1,Integer.MAX_VALUE,tianheQuery);
        UtilExcel.exportListByTemplate(response,tianheTB.getData(),Tianhe.class,"周报");
    }

    @PostMapping("taskLock")
    public McR taskLock(@RequestBody JSONObject param){
        thTbService.taskLock(param.getString("projectId"),param.getString("taskType"),param.getJSONArray("keys").toJavaList(String.class),param.getString("type").equals("add"));
        return McR.success();
    }

    @PostMapping("taskUpdate")
    public McR taskUpdate(@RequestBody JSONObject param){
        return thTbService.taskUpdate(param);
    }

    @PostMapping("saveTask")
    public McR saveTask(@RequestBody JSONObject param){
        thTbService.saveTask(param.getString("procInsId"));
        return McR.success();
    }

    @PostMapping("taskCancel")
    public McR taskCancel(@RequestBody JSONObject param){
        return thTbService.taskCancel(param);
    }

    @Autowired
    private YDClient ydClient;

    @GetMapping("/redirect-me")
    public ResponseEntity<Void> getRedirectUrl(String type,String state) {
        try {
            List<Map> list = (List<Map>) ydClient.queryData(YDParam.builder().formUuid("FORM-B40945246BBD4CD8A277DA8BE5949C1DJD3Z").searchCondition(
                    JSONObject.toJSONString(Arrays.asList(new YDSearch("selectField_m2mpdvul",type,"type", YDSearch.Type.TEXT_FIELD,YDSearch.Operator.EQ),
                            new YDSearch("selectField_m2mpdvum",state,"state", YDSearch.Type.TEXT_FIELD,YDSearch.Operator.EQ)))
            ).build(), YDConf.FORM_QUERY.retrieve_list).getData();
            if(list!=null&&list.size()>0){
                return ResponseEntity.status(302).header("Location", UtilMap.getString(UtilMap.getMap(list.get(0),"formData"),"textField_m2mpdvun")).build();
            }else{
                return ResponseEntity.status(302).header("Location", "https://z6eiye.aliwork.com/APP_D5TQPJQBB21M7BCJUMO5/workbench/FORM-0DCA7BDE2DF445358A242600041CE55E6TAP").build();
            }
        }catch (Exception e){
            e.printStackTrace();
            return ResponseEntity.status(302).header("Location", "https://z6eiye.aliwork.com/APP_D5TQPJQBB21M7BCJUMO5/workbench/FORM-0DCA7BDE2DF445358A242600041CE55E6TAP").build();
        }


    }

}
